package hegmonSharing;

/*
 * This is a replication of the model reported in Hegmon (1991) :
   Hegmon, M.
	1991	The Risks of Sharing and Sharing as Risk Reduction: Inter-household Food Sharing in Egalitarian Societies. 
	In Between Bands and States, edited by S. A. Gregg, pp. 309-332. Occasional Paper No. 9. Center for Archaeological 
	Investigation, Southern Illinois University Carbondale.
  
  The replication is performed by Sean Bergin, Arizona State University, August 2007.
  Replication of Burtsev and Turchin model Copyright (C) 1985 M. Hegmon ((original) Copyright (C) 2007 S. Bergin (replication)
  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 */

import java.awt.Color;
import java.util.ArrayList;
import java.util.Hashtable;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import cern.jet.random.Uniform;
import uchicago.src.reflector.ListPropertyDescriptor;
import uchicago.src.sim.analysis.DataSource;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.BasicAction;
import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.engine.SimInit;
import uchicago.src.sim.engine.SimModelImpl;
import uchicago.src.sim.gui.ColorMap;
import uchicago.src.sim.gui.DisplayConstants;
import uchicago.src.sim.gui.DisplaySurface;
import uchicago.src.sim.gui.Object2DDisplay;
import uchicago.src.sim.gui.Value2DDisplay;
import uchicago.src.sim.util.SimUtilities;


public class ShareModel extends SimModelImpl {
	  private int worldXSize=100;
	  private int worldYSize=100;
	  private int sharingType = 0;
	  private int year = 1932, endYear = 1972, startYear=1932;
	  private int groupSize, numHouses, groupTotal, houseNumber, groupTotalCurrent, restRowTotal, temp;
	  private int slowDown=0;//0 is yes, 1 is no
	  public int starvation =0;  //generic tally variables for  textual output
	  private Schedule schedule;
	  private ShareSpace sSpace;
	  private ArrayList houseList;
	  private DisplaySurface displaySurf;
	  private OpenSequenceGraph graph;
	  private BufferedWriter output;
	  int[][] sharingPoolingGroup;
	  int[][] sharingRestrictedGroup;
	  int[][] sharingGroupStatus;
	  int[][]sumGroupHarvestNeed;
	  int[][]sumGroupHarvestExtra;

	  public  ShareModel (){ 
		    
	    Hashtable h = new Hashtable();// drop down menus
	    h.put(new Integer(0), "None");
	    h.put(new Integer(1), "Pooling");
	    h.put(new Integer(2), "Restricted");
	    h.put(new Integer(3), "AllTypes");
	    ListPropertyDescriptor pd1 = new ListPropertyDescriptor("SharingType", h);
	    descriptors.put("SharingType", pd1);
	    
	    Hashtable h1 = new Hashtable();
	    h1.put(new Integer(0), "Normal");
	    h1.put(new Integer(1), "Fast");

	    ListPropertyDescriptor pd2 = new ListPropertyDescriptor("Speed", h1);
	    descriptors.put("Speed", pd2);
        
	  }
	  public String getName(){
	    return "Hegmon's Risks of Sharing";
	  }

	  public void setup(){
		  System.out.println("Running setup");
		    sSpace = null;
		    houseList = new ArrayList();
		    schedule = new Schedule(1);
		    year = 1932;  

		 //Tear down Displays
		    if (displaySurf != null){
		    	displaySurf.dispose();
		    }
		    displaySurf = null;
		    
		  if (graph != null){
			  graph.dispose();
	      }
		  graph = null;
		  // Create Displays
		  displaySurf = new DisplaySurface(this, "Hegmon's Agricultural Sharing Simulation");
		  graph = new OpenSequenceGraph("Household / Group Survival",this);
		    // Register Displays
		    registerDisplaySurface("Sharing ", displaySurf);
	  }

	  public void begin(){
	    buildModel();
	    buildSchedule();
	    buildDisplay();
	    displaySurf.display();
	    graph.display();
	  }

	  public void buildModel(){
		  System.out.println("Running BuildModel");
		  sSpace = new ShareSpace(worldXSize, worldYSize);
		  groupTotalCurrent = groupTotal;
		  houseNumber = 0;
		  year = startYear;
		  sSpace.setYear(year);
		  if(sharingType == 0){// no sharing
			  System.out.println("No Sharing will occur");
			  if(numHouses==0){
				  numHouses=Uniform.staticNextIntFromTo(2, 40);
				  System.out.println("Number of Houses was not chosen,"+numHouses+" households will be used");
		  }
		    for(int i = 0; i < numHouses; i++){
		        addNewHouse();
		     }
		  }  
		  if(sharingType ==1){//pooling resources
			  System.out.println("Groups will pool resources");
			  
			  if (groupTotal == 0){
				  System.out.println("Group Number was not chosen, a random number will be chosen");
				  groupTotal = Uniform.staticNextIntFromTo(2, 40);
				  System.out.println("Group number is "+groupTotal);
			  }
			  if (groupSize == 0){
				  System.out.println("Group Size was not chosen, a random number will be chosen");
				  groupSize = Uniform.staticNextIntFromTo(2, 40);
				  System.out.println("Group Size is "+groupSize);
			  }
			  sharingGroupStatus = new int [groupTotal][1];//used to count the number of surviving groups
			  		for(int i = 0; i < groupTotal; i++){
			  			sharingGroupStatus [i][0] = 1;
			  		}
			  		addSharePoolingHouses();
		    }
		  if(sharingType ==2){// restricted sharing
			  System.out.println("Groups will practice restricted sharing of resources");
			  if (groupTotal == 0){
				  System.out.println("Group Number was not chosen, a random number will be chosen");
				  groupTotal = Uniform.staticNextIntFromTo(2, 40);
				  System.out.println("Group number is "+groupTotal);
			  }
			  if (groupSize == 0){
				  System.out.println("Group Size was not chosen, a random number will be chosen");
				  groupSize = Uniform.staticNextIntFromTo(2, 40);
				  System.out.println("Group Size is "+groupSize);
			  }
			  restRowTotal = (groupTotal*groupSize);
			  addShareRestrictedHouses();  
		    }
		  if(sharingType ==3){// multiple types of sharing will occur
			  System.out.println("A Group will pool resources, a group will practice a restricted sharing startegy, and a group will not share");
			  if (groupTotal == 0){
				  System.out.println("Group Number was not chosen, a random number will be chosen");
				  groupTotal = Uniform.staticNextIntFromTo(2, 40);
				  System.out.println("Group number is "+groupTotal);
			  }
			  if (groupSize == 0){
				  System.out.println("Group Size was not chosen, a random number will be chosen");
				  groupSize = Uniform.staticNextIntFromTo(2, 40);
				  System.out.println("Group Size is "+groupSize);
			  }
			    sharingGroupStatus = new int [groupTotal][1];//used to count the number of surviving groups for pooling
		  		for(int i = 0; i < groupTotal; i++){
		  			sharingGroupStatus [i][0] = 1;
		  		}
		  		restRowTotal = (groupTotal*groupSize);
		  		addShareRestrictedHouses(); 
		  		addSharePoolingHouses();
			    for(int i = 0; i < restRowTotal; i++){
			        addNewHouse();
			     }			  
		  }
		  if (sharingType>=4){
			  System.out.println("Invalid Share Type Chosen");
			  System.out.println("No Sharing will occur");
			  sharingType = 0;
		  }
		  	System.out.println("NumHouseholds  "+ houseNumber);
		  	int x =groupTotal;
		  	if(sharingType==3){
			  x=groupTotal*2;
		  }
		  System.out.println("Num of Groups "+ x);
		  System.out.println("Households per group "+groupSize);
	  }

	  
	  public void buildSchedule(){
		    System.out.println("Running BuildSchedule");

		    class ShareStep extends BasicAction {
		      public void execute() {
		    	  graph.step();
		    	  int totalAgentsAlive = numberofCurrentHousesForType(0)+numberofCurrentHousesForType(1)+numberofCurrentHousesForType(2);
		    	  if (year ==endYear || totalAgentsAlive ==0){
		    		  stop();
		    	  }
		    	  if(slowDown == 0){
		    		  System.out.println("Running SlowDown");
		    		  for(int i=0; i<100000000; i++){  //the title makes it obvious, but this is here to slow down the program
		    		  slowDown();
		    		  }
		    	  } 
		    		  sSpace.setYear(year);
		    		  if(sharingType == 0){
		    			  for(int i =0; i < houseList.size(); i++){
		    				  ShareAgent house = (ShareAgent)houseList.get(i);
		    				  house.step(); 
		    			  }
		    		  }
		    		  if(sharingType == 1){
		    			  for(int i =0; i < houseList.size(); i++){ //all eat or are hungry
		    				  ShareAgent house = (ShareAgent)houseList.get(i);
		    	 			 	house.step(); 
		    	 		  	}
		    			  sharePooling();
		    		  }
		    		  if(sharingType == 2){
		    			  for(int i =0; i < houseList.size(); i++){ //all eat or are hungry
		    				  ShareAgent house = (ShareAgent)houseList.get(i);
		    	 			 	house.step(); 
		    	 		  	}
		    			  shareRestricted();
		    		  }
		    		  if(sharingType==3){
		    			  for(int i =0; i < houseList.size(); i++){ //all eat or are hungry
		    				  ShareAgent house = (ShareAgent)houseList.get(i);
		    	 			 	house.step(); 
		    	 		  	}
		    			  sharePooling();
		    			  shareRestricted();
		    		  }
		    		  displaySurf.updateDisplay();
		    		  System.out.println("  Houses or Groups : " + numberOfCurrentHouses() + ":  The Current Year is : " + year + ": starvation : "+ starvation);	    		  
			    	  year++;
			    	  starvation=0;
		     }   
		    }
		    schedule.scheduleActionAtInterval(1, new ShareStep());
	  }
	  public void buildDisplay(){
		  System.out.println("Running BuildDisplay");
		  
		  ColorMap map = new ColorMap();
		  map.mapColor(0, Color.white);
		  
		  Value2DDisplay displayGrid = new Value2DDisplay(sSpace.getCurrentGridSpace(), map);
		  Object2DDisplay displayAgents = new Object2DDisplay(sSpace.getCurrentAgentSpace());
		  displayAgents.setObjectList(houseList);
		  displaySurf.addDisplayableProbeable(displayGrid, "Grid");
		  displaySurf.addDisplayableProbeable(displayAgents, "Agents");
		  
		  int yRange = 1;
		  if(sharingType == 1){
			  yRange = numHouses;
		  }
		  if(sharingType == 2 || sharingType ==3){
			  yRange = groupTotal;
		  }
		  
		  if(sharingType ==0 ||sharingType ==1||sharingType==2){
		    graph.addSequence("Num. Households or Sharing Groups", new Sequence() {
		        public double getSValue() {
		          return (double)numberOfCurrentHouses();
		        }
		      });
		  }
		  if(sharingType ==3){
			  graph.addSequence("Num. Households Not Sharing", new Sequence() {
			        public double getSValue() {
			          return (double)numberofCurrentHousesForType(0);
			        }
			      });
			  graph.addSequence("Num. Households Pooling Resources", new Sequence() {
			        public double getSValue() {
			          return (double)numberofCurrentHousesForType(1);
			        }
			      });
			  graph.addSequence("Num. Households Using Restricted Sharing", new Sequence() {
			        public double getSValue() {
			          return (double)numberofCurrentHousesForType(2);
			        }
			      });
		  }
		      graph.setXRange(0, 100);
		      graph.setYRange(0, yRange);//?!
		      graph.setAxisTitles("time", "Houses/Groups");
	  }

	  public Schedule getSchedule(){
	    return schedule;
	  }

	  public void slowDown(){
		  for(long i=0;i>1000000000; i++){
			  for(long j=0;j>1000000000; j++){
				  for(long m=0;m>1000000000; m++){
					  int k = (50*40*20*30*10);
					  ShareAgent house = (ShareAgent)houseList.get(0);
					  house.areYouHungry();
				  }
			  }
		  }
	  }
	  
	  public String[] getInitParam(){
	    String[] initParams = {"EndYear","StartYear","NumHouseholds" , "WorldXSize", "WorldYSize", "SharingType", "HouseholdsPerGroup", "NumberOfGroups", "Speed"};
	    return initParams;
	  }
	  			/*Initial Parameters*/
	  public int getStartYear(){
		  return startYear;
	  }
	  public void setStartYear(int y){
		  startYear=y;
	  }
	  public int getEndYear(){
		  return endYear;
	  }
	  public void setEndYear(int y){
		  endYear=y;
	  }
	  public int getSpeed(){
		  return slowDown;
	  }
	  public void setSpeed(int slow){
		  slowDown = slow;
	  }
	  public int getNumHouseholds(){
		  return numHouses;
	  }
	  public void setNumHouseholds(int nh){
		  numHouses = nh;
	  }
	  public int getWorldXSize(){
	    return worldXSize;
	  }
	  public void setWorldXSize(int wxs){
	    worldXSize = wxs;
	  }
	  public int getWorldYSize(){
	    return worldYSize;
	  }
	  public void setWorldYSize(int wys){
	    worldYSize = wys;
	  }
	  public int getSharingType(){
		    return sharingType;
		  }
	  public void setSharingType(int share){
		  sharingType = share;
		  }	  
	  public int getHouseholdsPerGroup(){
		    return groupSize;
		  }
	  public void setHouseholdsPerGroup(int group){
		  groupSize = group;
		  }
	  public int getNumberOfGroups(){
		    return groupTotal;
		  }
	  public void setNumberOfGroups(int group){
		  groupTotal = group;
		  } 
	  public static void main(String[] args) {
	    SimInit init = new SimInit();
	    ShareModel model = new ShareModel();
	    init.loadModel(model, "", false);
	  }
	  
	  public int numberofCurrentHousesForType(int type){
		  int total = 0;
		  for(int i =0; i < houseList.size(); i++){
			  ShareAgent house = (ShareAgent)houseList.get(i);
			  if(house.getSharingType() == type){
				  int alive = house.getAliveStatus();
				  if (alive == 1){
					  total++;
				  }
			  }
		  }
		  return total;
	  }
	  public int numberOfCurrentHouses(){
		  int total = 0;
		  if(sharingType ==0){//no sharing counts houses
			  for(int i =0; i < houseList.size(); i++){
				  ShareAgent house = (ShareAgent)houseList.get(i);
				  int alive = house.getAliveStatus();
				  if (alive == 1){
					  total++;
				  }
			  }
		  }
		  if(sharingType==1){//groups pooling their resources
			  for(int i = 0; i < groupTotal; i++){
		  			if(sharingGroupStatus [i][0] ==1){
		  				total++;
		  			}
		  		}
		  }
		  if(sharingType==2){
			  for(int i =0; i < houseList.size(); i++){
				  ShareAgent house = (ShareAgent)houseList.get(i);
				  int alive = house.getAliveStatus();
				  if (alive == 1){
					  total++;
				  }
			  }
			  total = total/groupSize; // only want to display the group status, not individuals
		  }	
		  
		  return total;
	  }
	  
	  private void addNewHouse(){ 
		    //System.out.println("HouseNumber when no share houses start to be added :" +houseNumber);
		  	int x = Uniform.staticNextIntFromTo(1, worldXSize);
		  	int y = Uniform.staticNextIntFromTo(1, worldYSize);
		  	while(sSpace.isCellOccupied(x, y) == true){
		  		 x = Uniform.staticNextIntFromTo(1, worldXSize);
			  	 y = Uniform.staticNextIntFromTo(1, worldYSize);	
		  	}
		    ShareAgent a = new ShareAgent(sSpace, this,0);
		    houseList.add(a);
		    a.setHouseNumber(houseNumber);
		    a.setAliveStatus(1);
		    a.setX(x);
		    a.setY(y);
		    houseNumber++;
		  }
	  
	  private void  addSharePoolingHouses (){
		  sharingPoolingGroup = new int [groupTotal] [groupSize+1];// contains household ID numbers for each group and first row for harvest
		  for (int i = 0; i< groupTotal; i++){
			  sharingPoolingGroup [i][0]= 266*groupSize;// initial storage of each group member is put into the storage
			  for (int j = 1; j< groupSize+1; j++){
				  	int x = Uniform.staticNextIntFromTo(1, worldXSize);
				  	int y = Uniform.staticNextIntFromTo(1, worldYSize);
				  	while(sSpace.isCellOccupied(x, y) == true){
				  		 x = Uniform.staticNextIntFromTo(1, worldXSize);
					  	 y = Uniform.staticNextIntFromTo(1, worldYSize);	
				  	}
				    ShareAgent a = new ShareAgent(sSpace, this, 1);
				    houseList.add(a);
				    a.setAliveStatus(1);
				    a.setHouseNumber(houseNumber);
				    a.setGroupNumber(i);
				    a.setX(x);
				    a.setY(y);
				    sharingPoolingGroup [i][j]= houseNumber;
				    //System.out.println("houseNumber = "+houseNumber +"groupNumber = "+  i);
				    houseNumber++;
			  }
		  }
		  numHouses = houseNumber;
		  //System.out.println("HouseNumber after pooling houses have been added :" +houseNumber);
	  }
	  
	  private void  addShareRestrictedHouses (){
		  //System.out.println("HouseNumber when share restricted houses start to be added :" +houseNumber);
		  int groupNumber = 1;
		  sharingRestrictedGroup = new int [restRowTotal] [4];//essentially contains a row with group number, house number, hungry?, extra harvest
		  sumGroupHarvestNeed = new int [groupTotal] [1];
		  sumGroupHarvestExtra = new int [groupTotal] [1];
		  //sharingRestrictedGroup rows: 0 groupnumber, 1 housenumber, 2 hungry?, 3 amount in storage
		  for (int i = 0; i< restRowTotal; i++){
			  sharingRestrictedGroup [i][0]= groupNumber;
			  sharingRestrictedGroup [i][2]= 0;
			  sharingRestrictedGroup [i][3]= 0;
			  int x = Uniform.staticNextIntFromTo(1, worldXSize);
			  int y = Uniform.staticNextIntFromTo(1, worldYSize);
			  	while(sSpace.isCellOccupied(x, y) == true){
			  		 x = Uniform.staticNextIntFromTo(1, worldXSize);
				  	 y = Uniform.staticNextIntFromTo(1, worldYSize);	
			  	}
			  ShareAgent a = new ShareAgent(sSpace, this,2);
			  houseList.add(a);
			  a.setAliveStatus(1);
			  a.setHouseNumber(houseNumber);
			  a.setGroupNumber(groupNumber);
			  a.setX(x);
			  a.setY(y);
			  sharingRestrictedGroup [i][1]= houseNumber;
			  houseNumber++;
			  System.out.println("houseNumber: " +houseNumber+" groupNumber "+ sharingRestrictedGroup [i][0]);
			  if ((i+1) == groupSize*groupNumber ){ // change the group number every revolution
				  groupNumber++;
			  }  
		  }
		  numHouses = houseNumber;
		  //System.out.println("HouseNumber after restricted houses have been added : " +houseNumber);
	  }
	  
	  public void  sharePoolingHarvest (int groupNumber, int harvest){
		  int x = sharingPoolingGroup[groupNumber][0];
		  sharingPoolingGroup[groupNumber][0]= x + harvest;
	  }
	  
	  public void groupFailure (int x){
		  //System.out.println("Group Number " + x + ":  died");
		  sharingGroupStatus [x][0] = 0; //row represents the group number
	  }
	  
	  public void houseStarved(int n){
		  starvation++;
		  numHouses--;
		  //System.out.println("HouseNumber " + n + ":  died");
	  }
	  
	  public void updateRestrictStatus(int house, int hungry, int storage){
		  for (int i=0; i< restRowTotal; i++){
			  if (house == sharingRestrictedGroup[i][1]){
				  sharingRestrictedGroup [i][2] = hungry;// hungry 1 yes, 2 no, 3 dead
			  	  sharingRestrictedGroup [i][3] = storage; // storage amt
			  }
		  } 
	  }
	  public void updateRestrictedGroupHarvest(){

		  for (int i=0; i<groupTotal;i++){//zero everythign out
			  sumGroupHarvestNeed [i][0]=0;
			  sumGroupHarvestExtra[i][0]=0;
		  }
		  for (int i = 0; i< restRowTotal; i++){// i= row	
			  int groupNumber = sharingRestrictedGroup[i][0];
			  if (sharingRestrictedGroup [i][2] == 1){ // if hungry
				  sumGroupHarvestNeed [groupNumber-1][0] = (1017 - sharingRestrictedGroup [i][3])+ sumGroupHarvestNeed [groupNumber-1][0];
			  }// group number-1 is the row number for need array
			  else if (sharingRestrictedGroup [i][2]==2){// if not hungry
				  sumGroupHarvestExtra [groupNumber-1][0] = sharingRestrictedGroup [i][3]+ sumGroupHarvestExtra[groupNumber-1][0];
			  }//group number-1 is the row number for extra array
		  }
		  
	  }
	  
	  public void sharePooling(){
		  for (int i = 0; i< groupTotal; i++){
			  int distribute = sharingPoolingGroup [i][0]; // groups total resources
			  sharingPoolingGroup [i][0] = 0; // zero it out so that no resources are available from past year, except if statement below
			  int shares = distribute/groupSize;
			  if(shares>1017){ //they have joint storage in case there is extra
				  sharingPoolingGroup [i][0] =  (shares-1017)*groupSize; // put the extra into the group storage
				  shares=1017;
			  }
			  for (int j = 1; j< groupSize+1; j++){//distribute shares  
				int n = sharingPoolingGroup [i][j];
				ShareAgent house = (ShareAgent)houseList.get(n);
				System.out.println(":HouseNumber: "+n+" :received: "+shares);
				house.poolingDistribute(shares);
			  }
		  }
	  }
	 
	  public void shareRestricted(){ 
		  updateRestrictedGroupHarvest();//sums up the need and extra resources for groups  
		  for (int i=0; i<groupTotal;i++){
			  int groupExtra = sumGroupHarvestExtra[i][0]-sumGroupHarvestNeed [i][0];
			  int needToGive = sumGroupHarvestNeed [i][0];
			  int canGive = sumGroupHarvestExtra[i][0];
			  //get group information
			  int given = 0;
			  int group=i+1;
			  if (groupExtra>0){
				  //take resources from those who have them
				  while(given<needToGive){
					  //System.out.println("in greater than 0 while statement "+given+ " given "+needToGive+" need to give"); 
					  for (int j=0; j< restRowTotal; j++){// j is row number
						  if(given<needToGive){ 
							  if(group == sharingRestrictedGroup[j][0]&& sharingRestrictedGroup[j][2]==2 && sharingRestrictedGroup[j][3]>0){
								  //group same and not hungry and has resources
								  sharingRestrictedGroup[j][3]= sharingRestrictedGroup[j][3]-1;//take away a resource
								  given=given+1;
							  }
						  }	  
					  }
				  }
				  //give resources & update new storage
				  for (int j=0; given<needToGive && j< restRowTotal; j++){
					  if(group == sharingRestrictedGroup[j][0]&& sharingRestrictedGroup[j][2]==1){
						  // no one is hungry or has any more in storage as long as they were hungry
						  sharingRestrictedGroup[j][2]=2;
						  sharingRestrictedGroup[j][3]=0;
					  }
				  }
			  }
			  else if (groupExtra==0){
				  for (int j=0; j< restRowTotal; j++){
					  if (group == sharingRestrictedGroup[j][0]){
						  	sharingRestrictedGroup[i][3]=0;// no storage anymore
						  	sharingRestrictedGroup[i][2]=2;// not hungry anymore
					  }
				  }
			  }
			  else if (groupExtra<0){
				  //give resources till cannot give anymore, households may still be hungry afterwards
				  while (given<canGive){
					  //System.out.println("in less than 0 while statement "+given+ " given "+canGive+" can give"); 
					  for (int j=0; j< restRowTotal; j++){// j is row number
						  if(given<canGive){ 
							  if(group == sharingRestrictedGroup[j][0]&& sharingRestrictedGroup[j][2]==1){// group same and is hungry  
								  sharingRestrictedGroup[j][3]= sharingRestrictedGroup[j][3]+1; //give it a resource
								  given=given+1;
								  if(sharingRestrictedGroup[j][3]>=1017 ){
									  sharingRestrictedGroup[j][2]=2;// if household fed then it needs to be made no longer hungry
								  }
							  }
						  }
				      }	  
			      }
				  // take resources & update new storage
				  for (int j=0; given<needToGive && j< restRowTotal; j++){
					  if(i+1 == sharingRestrictedGroup[j][0]){// no one has any more in storage
						  sharingRestrictedGroup[j][3]=0;
					  }
				  }
				   
			  }
		  }	  
		  updateNewStorageNumbers(); 
	  }
		  
	  public void updateNewStorageNumbers(){
		  for(int i = 0; i< restRowTotal; i++){
			  ShareAgent house = (ShareAgent)houseList.get(sharingRestrictedGroup[i][1]);
			  if(sharingRestrictedGroup[i][2] == 1){
				  house.setMalnourishedState();
			  }
			  house.setHarvestStorage(sharingRestrictedGroup[i][3]);
		  }
	  }
		  
}	  
